/***********************************************************
** (C) KEISUKE KONDO
** 
** Release Date: October 24, 2015
** Last Updated: October 28, 2015
** Version: 1.10
** 
** 
** [Contact]
** Email: kondo-keisuke@rieti.go.jp
** URL: https://sites.google.com/site/keisukekondokk/
***********************************************************/

capture program drop spgen
program spgen, sortpreserve rclass
	version 11
	syntax varlist [if] [in], /*
			*/ lat(varname) /*
			*/ lon(varname) /*
			*/ swm(string) /*
			*/ dist(real) /*
			*/ [ /*
			*/ Order(real 1) /*
			*/ NOSTD /*
			*/ DMS /*
			*/ APProx /*
			*/ Detail ]

	/*Variables*/
	local vY `varlist'
	local swmtype = substr("`swm'",1,3)
	marksample touse
	markout `touse' `lat' `lon'
	
	/*Check Variable*/
	if( strpos("`vY'"," ") > 0 ){
		display as error "Multiple variables are not allowed."
		exit 198
	}
	
	/*Check Latitude Range*/
	if( `lat' <= -90  | `lat' >= 90 ){
		display as error "lat() must be within -90 to 90."
		exit 198
	}
	
	/*Check Longitude Range*/
	if( `lon' <= -180  | `lon' >= 180 ){
		display as error "lon() must be within -180 to 180."
		exit 198
	}
	
	/*Check Spatial Weight Matrix*/
	if( "`swmtype'" != "bin" & "`swmtype'" != "exp" & "`swmtype'" != "pow" ){
		display as error "swm() must be one of bin, exp, and pow."
		exit 198
	}

	/*Check Distance Decay Parameter of Spatial Weight Matrix*/
	if( "`swmtype'" == "bin" ){
		local dd = . /*not used*/
	}
	else if( "`swmtype'" == "exp" ){
		local dd = real(substr("`swm'",strpos("`swm'","exp")+length("exp")+1,.))
		if( `dd' <= 0 ){
			display as error "Distance-decay parameter must be more than 0."
			exit 198
		}
		else if( `dd' == . ){
			display as error "Numerical type is expected for distance-decay parameter."
			exit 198
		}
	}
	else if( "`swmtype'" == "pow" ){
		local dd = real(substr("`swm'",strpos("`swm'","pow")+length("pow")+1,.))
		if( `dd' <= 0 ){
			display as error "Distance-decay parameter must be more than 0."
			exit 198
		}
		else if( `dd' == . ){
			display as error "Numerical type is expected for distance-decay parameter."
			exit 198
		}
	}
	
	/*Check Parameter Range*/
	if( `dist' <= 0 ){
		display as error "dist() must be more than 0."
		exit 198
	}
	
	/*Check Order*/
	if( `order' <= 0 ){
		display as error "order() must be more than 0."
		exit 198
	}
	capture confirm integer number `order'
	if( _rc != 0 ){
		display as error "order() must be integer."
		exit 7
	}
	
	/*Row Standardization of Spatial Weight Matrix*/
	local rowstd = 1
	if( "`nostd'" != "" ){
		local rowstd = 0
		if( `order' != 1 ){
			display as error "order() must be 1."
			exit 198
		}
	}
	
	/*Convert DMS Format to Decimal Format*/
	local fmdms = 0
	if( "`dms'" != "" ){
		local fmdms = 1
	}
	
	/*Approximation of Distance*/
	local appdist = 0
	if( "`approx'" != "" ){
		local appdist = 1
	}
	
	/*Detail*/
	local dispdetail= 0
	if( "`detail'" != "" ){
		local dispdetail = 1
	}
	
	/*Make Variable*/
	local error1 = 0
	local error2 = 0
	if( "`swmtype'" == "bin" ){
		capture confirm new variable splag`order'_`vY'_b, exact
		local error1 = _rc
	} 
	else if( "`swmtype'" == "exp" ){
		capture confirm new variable splag`order'_`vY'_e, exact
		local error1 = _rc
	} 
	else if( "`swmtype'" == "pow" ){
		capture confirm new variable splag`order'_`vY'_p, exact
		local error1 = _rc
	}
	
	/*Error Check*/
	if( `error1' == 110 ){
		display as error "Outcome variables already defined. Change variable name"
		exit 110
	}
	
	/*Call Mata Program*/
	mata: calcsplag("`vY'", "`lat'", "`lon'", "`fmdms'", "`swmtype'", `dist', `dd', `order', "`rowstd'", "`appdist'", "`dispdetail'", "`touse'")
	
	/*Return*/
	return add
	
	/*Label*/
	if( "`swmtype'" == "bin" ){
		if( `rowstd' == 1 ){
			label var splag`order'_`vY'_b "sptial lag, swm(bin), td=`dist', od=`order', row-standadized"
		}
		else if( `rowstd' == 0 ){
			label var splag`order'_`vY'_b "sptial lag, swm(bin), td=`dist', od=`order', no row-standadized"
		}
		display as txt "{bf:splag`order'_`vY'_b} is generated in the dataset."
	} 
	else if( "`swmtype'" == "exp" ){
		if( `rowstd' == 1 ){
			label var splag`order'_`vY'_e "sptial lag, swm(exp), td=`dist', dd=`dd', od=`order', row-standadized"
		}
		else if( `rowstd' == 0 ){
			label var splag`order'_`vY'_e "sptial lag, swm(exp), td=`dist', dd=`dd', od=`order', no row-standadized"		
		}
		display as txt "{bf:splag`order'_`vY'_e} is generated in the dataset."
	} 
	else if( "`swmtype'" == "pow" ){
		if( `rowstd' == 1 ){
			label var splag`order'_`vY'_p "sptial lag, swm(pow), td=`dist', dd=`dd', od=`order', row-standadized"
		}
		else if( `rowstd' == 0 ){
			label var splag`order'_`vY'_p "sptial lag, swm(pow), td=`dist', dd=`dd', od=`order', no row-standadized"
		}
		display as txt "{bf:splag`order'_`vY'_p} is generated in the dataset."
	} 
end


/*Getis-Ord G*(d) Calculation*/
version 11
mata:
void calcsplag(vY, lat, lon, fmdms, swmtype, dist, dd, order, rowstd, appdist, dispdetail, touse)
{
	/*Check format of latitude and longitude*/
	if( fmdms == "1" ){
		printf("{txt}...Converting DMS format to decimal format\n")
		convlonlat2decimal(lat, lon, touse, &vlat, &vlon)
	} 
	else {
		st_view(vlat, ., lat, touse)
		st_view(vlon, ., lon, touse)
	}
	latr = ( pi() / 180 ) * vlat; 
	lonr = ( pi() / 180 ) * vlon; 
	
	/*Make Variable*/
	st_view(vy, ., vY, touse)
	cN = rows(vlon)
	mD = J(cN,cN,0)

	/*Make Distance Matrix using Vincenty (1975) */
	if( appdist == "1" ){
		/*Simplified Version of Vincenty Formula*/
		calcdist2(cN, latr, lonr, &mD)
	} 
	else {
		/*Vincenty Formula*/
		calcdist1(cN, latr, lonr, &mD)
	}
	
	/*Spatial Weight Matrix*/
	if( swmtype == "bin" ){
		mTemp = ( mD :< dist )
		_diag(mTemp,0)
		if( rowstd == "1" ){
			mW = mTemp :/ rowsum(mTemp)
		} 
		else if( rowstd == "0" ){
			mW = mTemp
		}
	} 
	else if( swmtype == "exp" ){
		mTemp = ( mD :< dist ) :* exp( - dd :* mD ) 
		_diag(mTemp,0)
		if( rowstd == "1" ){
			mW = mTemp :/ rowsum(mTemp)
		} 
		else if( rowstd == "0" ){
			mW = mTemp
		}
	} 
	else if( swmtype == "pow" ){
		mTemp = ( mD :< dist ) :* ( mD ):^(-dd)
		_diag(mTemp,0)
		if( rowstd == "1" ){
			mW = mTemp :/ rowsum(mTemp)
		} 
		else if( rowstd == "0" ){
			mW = mTemp
		}
	}
	
	/*Order of Spatial Lag*/
	mTemp = I(cN)
	for(i=1; i<=order; ++i){
		mTemp = mW * mTemp
	}
	
	/*Spatial Lag*/
	vWY = mTemp * vy;
	
	/*Summary Statistics of Distance Matrix*/
	mD_L = lowertriangle(mD)
	vD = select(vech(mD_L), vech(mD_L):>0)
	dist_mean = mean(vD)
	dist_sd = sqrt(variance(vD))
	dist_min = min(vD)
	dist_max = max(vD)
	
	/*Display Summary Statistics of Distances*/
	printf("\n")
	if( appdist == "1" ){
		printf("{txt}Distance by simplified version of Vincenty formula\n")
	}
	else {
		printf("{txt}Distance by Vincenty formula\n")
	}
	if( dispdetail == "1" ){
		printf("\n")
		printf("{hline 13}{c TT}{hline 62} \n")
		printf("{txt}{space 12} {c |}        Obs.        Mean        S.D.        Min.         Max\n")
		printf("{hline 13}{c +}{hline 62} \n")
		printf("{txt}{space 4}Distance {c |}{res}  %10.0f  %10.3f  %10.3f  %10.3f  %10.3f\n", 
					rows(vD), dist_mean, dist_sd, dist_min, dist_max )	
		printf("{hline 13}{c BT}{hline 62} \n")
	}
	
	/*Return Resutls in Mata to Stata*/
	if( swmtype == "bin" ){
		st_store(., st_addvar("float","splag"+strofreal(order)+"_"+vY+"_b"), st_local("touse"), vWY)
	}
	if( swmtype == "exp" ){
		st_store(., st_addvar("float","splag"+strofreal(order)+"_"+vY+"_e"), st_local("touse"), vWY)
	}
	if( swmtype == "pow" ){
		st_store(., st_addvar("float","splag"+strofreal(order)+"_"+vY+"_p"), st_local("touse"), vWY)
	}

	/*rreturn command in Stata*/
	st_rclear()
	st_numscalar("r(dist_max)", dist_max)
	st_numscalar("r(dist_min)", dist_min)
	st_numscalar("r(dist_sd)", dist_sd)
	st_numscalar("r(dist_mean)", dist_mean)
	st_numscalar("r(od)", order)
	st_numscalar("r(dd)", dd)
	st_numscalar("r(td)", dist)
	st_numscalar("r(N)", cN)
	st_matrix("r(D)", mD_L)
	if( swmtype == "bin" ){
		st_global("r(swm)", "binary")
	} 
	else if( swmtype == "exp" ){
		st_global("r(swm)", "exponential")
	}
	else if( swmtype == "pow" ){
		st_global("r(swm)", "power")
	}
	if( appdist == "1" ){
		st_global("r(dist_type)", "approximation")
	}
	else if( appdist == "0" ){
		st_global("r(dist_type)", "exact")
	}
	if( rowstd == "1" ){
		st_global("r(swm_std)", "row-standardized")
	}
	else if( rowstd == "0" ){
		st_global("r(swm_std)", "no row-standardized")
	}
	st_global("r(varname)", vY)
	st_global("r(cmd)", "spgen")
}
end

/*Convert DMS format to Decimal Format*/
version 11
mata:
void convlonlat2decimal(lat, lon, touse, vlat, vlon)
{
	st_view(vlat_, ., lat, touse)
	st_view(vlon_, ., lon, touse)
	(*vlat) = floor(vlat_) :+ (floor((vlat_:-floor(vlat_)):*100):/60) :+ (floor((vlat_:*100:-floor(vlat_:*100)):*100):/3600)
	(*vlon) = floor(vlon_) :+ (floor((vlon_:-floor(vlon_)):*100):/60) :+ (floor((vlon_:*100:-floor(vlon_:*100)):*100):/3600)
}
end

/*Vincenty Formula*/
version 11
mata:
void calcdist1(cN, latr, lonr, mD)
{
	/*Variables*/
	mDist = J(cN,cN,0)
	a = 6378.137
	b = 6356.752314245
	f = (a-b)/a
	eps = 1e-12
	maxIt = 100
	
	/*Distance between i and j*/
	for( i=1; i<=cN; ++i ){
		for(j=i+1; j<=cN; ++j){
			U1 = atan( (1-f)*tan(latr[i]) )
			U2 = atan( (1-f)*tan(latr[j]) )
			L = lonr[i] - lonr[j]
			lam = L
			l1_lam = lam
			cnt = 0
			/*Iteration for Vincenty Formula*/
			do{
				numer1 = ( cos(U2)*sin(lam) )^2;
				numer2 = ( cos(U1)*sin(U2) - sin(U1)*cos(U2)*cos(lam) )^2;
				numer = sqrt( numer1 + numer2 );
				denom = sin(U1)*sin(U2) + cos(U1)*cos(U2)*cos(lam);
				sig = atan2( denom, numer );
				sinalp = (cos(U1)*cos(U2)*sin(lam)) / sin(sig);
				cos2alp = 1 - sinalp^2;
				cos2sigm = cos(sig) - ( 2*sin(U1)*sin(U2) ) / cos2alp;
				C = f/16 * cos2alp * ( 4+f*(4-3*cos2alp) );
				lam = L + (1-C)*f*sinalp*( sig+C*sin(sig)*( cos2sigm+C*cos(sig)*(-1+2*cos2sigm^2) ) );
				cri = abs( lam - l1_lam );
				l1_lam = lam;
				if( cnt++ > maxIt ){
					printf("{err}Convergence not achieved in Vincenty formula")
					printf("{err}region %f, \t region %f", i, j )
					exit(error(430))
				}
			}while( cri > eps )
			/*After Iteration*/
			u2 = cos2alp * ( (a^2-b^2)/b^2 )
			A = 1 + (u2/16384) * ( 4096 + u2*(-768+u2*(320-175*u2)) )
			B = u2/1024 * (256 + u2*(-128+u2*(74-47*u2)) )
			dsig = B*sin(sig)*( cos2sigm + 0.25*B*( cos(sig)*(-1+2*cos2sigm^2)-1/6*B*cos2sigm*(-3+4*sin(sig)^2)*(-3+4*cos2sigm) ) )
			mDist[i,j] = b*A*(sig-dsig)
		}
	}

	/*Return Distanc Matrix*/
	mDist = mDist + mDist'
	(*mD) = mDist
}
end

/*Simplified Version of Vincenty Formula*/
version 11
mata:
void calcdist2(cN, latr, lonr, mD)
{
	/*Distance between i and j*/
	A = J(cN,cN,1) :* lonr
	B = J(cN,cN,1) :* lonr'
	C = J(cN,cN,1) :* latr
	D = J(cN,cN,1) :* latr'
	difflonr = abs( A - B )
	numer1 = ( cos(D):*sin(difflonr) ):^2
	numer2 = ( cos(C):*sin(D) :- sin(C):*cos(D):*cos(difflonr) ):^2
	numer = sqrt( numer1 :+ numer2 )
	denom = sin(C):*sin(D) :+ cos(C):*cos(D):*cos(difflonr)
	mDist = 6378.137 :* atan2( denom, numer )

	/*Return Distanc Matrix*/
	(*mD) = mDist
}
end
